


#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "nrf_sdh_soc.h"
#include "nrf_sdh_ble.h"
#include "nrf_ble_gatt.h"
#include "nrf_ble_qwr.h"
#include "app_timer.h"
#include "ble_nus.h"
#include "app_uart.h"
#include "app_util_platform.h"
#include "bsp_btn_ble.h"
#include "nrf_pwr_mgmt.h"

#if defined (UART_PRESENT)
#include "nrf_uart.h"
#endif
#if defined (UARTE_PRESENT)
#include "nrf_uarte.h"
#endif

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "nrf_delay.h"

#define APP_BLE_CONN_CFG_TAG            1                                           /**< A tag identifying the SoftDevice BLE configuration. */

#define DEVICE_NAME                     "Nordic_UART-warmer2"                               /**< Name of device. Will be included in the advertising data. */
#define NUS_SERVICE_UUID_TYPE           BLE_UUID_TYPE_VENDOR_BEGIN                  /**< UUID type for the Nordic UART Service (vendor specific). */

#define APP_BLE_OBSERVER_PRIO           3                                           /**< Application's BLE observer priority. You shouldn't need to modify this value. */

#define APP_ADV_INTERVAL                64                                          /**< The advertising interval (in units of 0.625 ms. This value corresponds to 40 ms). */

#define APP_ADV_DURATION                18000                                       /**< The advertising duration (180 seconds) in units of 10 milliseconds. */

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(20, UNIT_1_25_MS)             /**< Minimum acceptable connection interval (20 ms), Connection interval uses 1.25 ms units. */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(75, UNIT_1_25_MS)             /**< Maximum acceptable connection interval (75 ms), Connection interval uses 1.25 ms units. */
#define SLAVE_LATENCY                   0                                           /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(4000, UNIT_10_MS)             /**< Connection supervisory timeout (4 seconds), Supervision Timeout uses 10 ms units. */
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000)                       /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000)                      /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                           /**< Number of attempts before giving up the connection parameter negotiation. */

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define UART_TX_BUF_SIZE                256                                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                256                                         /**< UART RX buffer size. */


BLE_NUS_DEF(m_nus, NRF_SDH_BLE_TOTAL_LINK_COUNT);                                   /**< BLE NUS service instance. */
NRF_BLE_GATT_DEF(m_gatt);                                                           /**< GATT module instance. */
NRF_BLE_QWR_DEF(m_qwr);                                                             /**< Context for the Queued Write module.*/
BLE_ADVERTISING_DEF(m_advertising);                                                 /**< Advertising module instance. */

static uint16_t   m_conn_handle          = BLE_CONN_HANDLE_INVALID;                 /**< Handle of the current connection. */
static uint16_t   m_ble_nus_max_data_len = BLE_GATT_ATT_MTU_DEFAULT - 3;            /**< Maximum length of data (in bytes) that can be transmitted to the peer by the Nordic UART service module. */
static ble_uuid_t m_adv_uuids[]          =                                          /**< Universally unique service identifier. */
{
    {BLE_UUID_NUS_SERVICE, NUS_SERVICE_UUID_TYPE}
};


/**********************************************************************************************************************************************/
char *DID = "SBWXXX";

float slv = 0.5;  //skin low value
float shv = 5.0;  //skin high vlaue
float alv = 0.5;  //air low value
float ahv = 5.0;  //air high value


uint8_t adt = 10; //alarm delay time
uint8_t amt = 20; //alarm Mute Time
uint8_t Hadt = 5; //High priority alarm delay time

float weight_dt = 4;
uint32_t weight_ticks = 0;
uint32_t weight_send_ticks = 0;
uint32_t bmd_ticks = 0;
uint32_t did_check_ticks = 0;
bool did_check_flag = true;
uint8_t hp_flag = 1;

float blink_dt = 1.0;
float beep_dt = 0.5;
bool blink_led = false;
bool buz_on = false;
uint8_t buz_cnt = 0;

uint32_t buz_ontime = 0;
uint32_t buz_offtime = 0;

uint32_t led_ontime = 0;
uint32_t led_offtime = 0;
bool blink_state = true;
bool buz_state = true;
uint8_t ble_live = 0;
uint8_t relayStatus = 0;

#define ADMITTED 1
#define DISCHARGED 2


uint8_t baby_status = 3;
//bool ble_connected = false;
bool ble_fail_process = true;
bool ble_flag = true;

uint8_t mode = 2; //default - skin mode
uint8_t real_mode;
uint8_t real_htr;
uint8_t spo_send_ready = 0;
uint8_t bsf_flag = 1;


#define LED                   NRF_GPIO_PIN_MAP(1, 1)  //NRF_GPIO_PIN_MAP(0,13)// NRF_GPIO_PIN_MAP(1,1)
#define FLED                  NRF_GPIO_PIN_MAP(0, 14) //NRF_GPIO_PIN_MAP(0,13)// NRF_GPIO_PIN_MAP(1,1)
#define NLED                  NRF_GPIO_PIN_MAP(0, 13) //NRF_GPIO_PIN_MAP(0,13)// NRF_GPIO_PIN_MAP(1,1)
#define IR_PIN                NRF_GPIO_PIN_MAP(1, 2)
#define BUZZ                  NRF_GPIO_PIN_MAP(0, 12)
#define RELAY                 NRF_GPIO_PIN_MAP(0, 21)
#define AIR_MODE              1
#define SKIN_MODE             2
#define MANUAL_MODE           3
//
#define BABY_MODE             2
#define AIR_MODE              1
#define SKIN_MODE             2
#define NO_ERROR              0
#define AIR_TEMP_HIGH         2
#define AIR_TEMP LOW          3
#define BABY_TEMP_HIGH        4
#define BABY_TEMP_LOW         5
#define BABY_TEMP_COLD        6
#define BABY_TEMP_OVER_RANGE  7
#define COLD_STRESS_ON_SET    8
#define HEAT_STRESS_ON_SET    9
#define FEVER_ON_SET          10
#define BABY_INFECTION        11
#define CEHCK_SKIN_PROBE      12
#define STANDBY_MODE          0
#define NORMAL_MODE           4
#define NO_FAIL               0
#define AIR_PROBE_OPEN        3
#define SKIN_PROBE_OPEN       1
#define PERI_PROBE_OPEN       2
#define HEATER_FAIL           4
#define UNDEFINED_FAIL        5
#define WRITE_FAIL            6

/************Alarm Flags and ticks variables ************************/
struct allAlerts
{
  uint8_t bth_alert;
  uint8_t btl_alert;
  uint8_t bto_t1_alert;
  uint8_t bto_t3_alert;
  uint8_t btc_alert;
  uint8_t ath_alert;
  uint8_t atl_alert;
  //  uint8_t ato_alert;
  uint8_t cso_alert;
  uint8_t hso_alert;
  uint8_t fo_alert;
  uint8_t bi_alert;
  uint8_t csp_alert;
  uint8_t cs_alert;
  uint8_t spf_alert;
  uint8_t t2f_alert;
  uint8_t apf_alert;
  uint8_t hf_alert;
  uint8_t sph_alert;
  uint8_t spl_alert;
  uint8_t prh_alert;
  uint8_t prl_alert;
  uint8_t sd_alert;
  uint8_t lm_alert;
  uint8_t lp_alert;
  uint8_t bsf_alert;  //bmd sys fail
  uint8_t spo_alert;  //spo sys fail
  uint8_t blef_alert;
  uint8_t hp_alert; //when heater % is more than 50% up to more than 15min.
}
alert;

uint32_t send_ble_ticks = 0;
uint32_t flash_ticks = 0;
uint32_t bsf_ticks = 0;
uint32_t spo_ticks = 0;
uint32_t blef_ticks = 0;
uint32_t bth_ticks = 0;
uint32_t btl_ticks = 0;
uint32_t bto_t1_ticks = 0;
uint32_t bto_t3_ticks = 0;
uint32_t btc_ticks = 0;
uint32_t ath_ticks = 0;
uint32_t atl_ticks = 0;
//uint32_t ato_ticks = 0;
uint32_t cso_ticks = 0;
uint32_t hso_ticks = 0;
uint32_t fo_ticks = 0;
uint32_t bi_ticks = 0;
uint32_t csp_ticks = 0;
uint32_t cs_ticks = 0;

uint32_t spf_ticks = 0;
uint32_t t2f_ticks = 0;
uint32_t apf_ticks = 0;
uint32_t hf_ticks = 0;

uint32_t sph_ticks = 0;
uint32_t spl_ticks = 0;
uint32_t prh_ticks = 0;
uint32_t prl_ticks = 0;

uint32_t sd_ticks = 0;
uint32_t lm_ticks = 0;
uint32_t lp_ticks = 0;
uint32_t hp_ticks = 0;

uint32_t ble_fail_ticks = 0;

uint8_t temp_mode;
uint8_t bmd_mode = 0;
uint8_t Tfail_code = 0;
uint8_t fail_code = 0;

struct _temp_
{
float T1;
float T2;
float T3;
float deltaT;
float weight;
uint8_t htr;
uint8_t sens_fail;
uint8_t spo;
uint8_t prt;
}Actual;

struct SET_values
{
  float T1; //Baby set Temperature  (T1)
  float T3; //Air set Temperature   (T3)
  float Ul;
  float Ll;
  uint8_t htr;  //Air set Temperature   (H)
  uint8_t spo2_Ul;  //SPO2 set Value Upper Limit (SPO2)
  uint8_t spo2_Ll;  //SPO2 set Value Lower Limit (SPO2)
  uint8_t pRate_Ul; //Pulse rate set Value Upper Limit
  uint8_t pRate_Ll; //Pulse rate set Value Lower Limit
} set;



struct calibration
{
  float T1_skin;
  float T2_peri;
  float T3_air;
  float spo;
  float prt;
  float wt;
} cal;

//*******************************************************************************************************************************************//
bool htr_flag = false;
uint8_t htr_percentage(float , float );
void htr_control(void);
void send_warm2(uint8_t * ,size_t );

//*******************************************************************************************************************************************//
//*********************************************************************************************************
void ath_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"ATH\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void ath_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"ATH\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void atl_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"ATL\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void atl_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"ATL\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

///////////////////////////////////////////////////////////////////////////////////////
void bto_t3_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BTO\",\"s\":\"1\",\"V\":\"T3\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void bto_t3_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BTO\",\"s\":\"0\",\"V\":\"T3\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
void bto_t1_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BTO\",\"s\":\"1\",\"V\":\"T1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void bto_t1_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BTO\",\"s\":\"0\",\"V\":\"T1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
void bth_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BTH\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void bth_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BTH\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void btl_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BTL\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void btl_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BTL\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void btc_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BTC\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void btc_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BTC\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
void cso_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"CSO\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void cso_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"CSO\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void hso_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"HSO\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void hso_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"HSO\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void fo_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"FO\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void fo_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"FO\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void bi_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BI\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void bi_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"BI\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void csp_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"CSP\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void csp_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"CSP\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void cs_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"CS\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void cs_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"CS\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}
//////////////////////////////////////////////////////////////////////////////////////
void hp_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"HP\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void hp_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"HP\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}
//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
void sph_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"SPH\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void sph_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"SPH\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void spl_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"SPL\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void spl_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"SPL\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void prh_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"PRH\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void prh_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"PRH\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////
void prl_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"PRL\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void prl_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"PRL\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////
void spf_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"SPF\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void spf_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"SPF\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////
void t2f_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"T2F\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void t2f_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"T2F\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////
void apf_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"APF\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void apf_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"APF\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////
void hf_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"HF\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void hf_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"HF\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////
void sd_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"SD\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void sd_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"SD\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////
void lp_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"LP\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void lp_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"LP\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////
void lm_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"LM\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void lm_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"LM\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////
void bsf_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"BSF\",\"s\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

void bsf_close(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, 0, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"205\",\"AL\":\"BSF\",\"s\":\"0\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////
void ir_open(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, NULL, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"206\",\"IR\":\"1\"}", DID);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
  }
  //sendVitalParams_ble();
}

//**********************************************************************************************************************************
uint32_t app_timer_ms(uint32_t ticks)
{
  float numerator = ((float) APP_TIMER_CONFIG_RTC_FREQUENCY + 1.0f) * 1000.0f;
  float denominator = (float) APP_TIMER_CLOCK_FREQ;
  float ms_per_tick = numerator / denominator;

  uint32_t ms = ms_per_tick * ticks;

  return ms;
}

//*************************************************************************************************************
//#define TX_POWER_LEVEL                  (4)
static void tx_power_set(int8_t TX_POWER_LEVEL)
{
  ret_code_t err_code = sd_ble_gap_tx_power_set(BLE_GAP_TX_POWER_ROLE_ADV, m_advertising.adv_handle, TX_POWER_LEVEL);
  APP_ERROR_CHECK(err_code);
}
void sendVitalParams_ble(void)
{
  if (!did_check_flag)
  {
    uint8_t buff[128];
    memset(&buff, NULL, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"201\",\"AT\":\"%2.1f\",\"BT\":\"%2.1f\",\"PT\":\"%2.1f\",\"HT\":\"%d\",\"M\":\"%d\"}", DID, Actual.T3, Actual.T1, Actual.T2, Actual.htr, mode);
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
    send_ble_ticks = app_timer_cnt_get();
  }

}

void Send_open_alarms(void)
{
  if(alert.t2f_alert > 0)
       t2f_open();

    if(alert.apf_alert > 0)
      apf_open();

    if(alert.spf_alert > 0)
      spf_open();

    if(alert.hf_alert > 0)//Heater Fail
      hf_open();

    ///////////////////////////////////////////////////////////
    if(alert.ath_alert > 2)//1
      ath_open();

    if(alert.atl_alert > 2) //2
        atl_open();

    if(alert.bto_t3_alert > 2) //3
        bto_t3_open();

    /////////////////////////////////////////////

    if(alert.btc_alert > 2) //4
        btc_open();

    if(alert.bth_alert > 2)  //5
        bth_open();

    if(alert.btl_alert > 2) //6
        btl_open();

    if(alert.bto_t1_alert > 2) //7
        bto_t1_open();

    ///////////////////////////////////////////////
    if(alert.bi_alert > 2)//8
        bi_open();

    if(alert.cso_alert > 2)  //9
        cso_open();

    if(alert.csp_alert > 2) //10
        csp_open();

    if(alert.cs_alert > 2) //11
        cs_open();

    if(alert.fo_alert > 2) //12
        fo_open();

    if(alert.hso_alert > 2) //13
        hso_open();
}//Send_open_alarms
void close_spo_alarms(void)
{
  if (alert.sph_alert != 0)
  {
    sph_close();
    alert.sph_alert = 0;
  }

  if (alert.spl_alert != 0)
  {
    spl_close();
    alert.spl_alert = 0;
  }

  if (alert.prh_alert != 0)
  {
    prh_close();
    alert.prh_alert = 0;
  }

  if (alert.prl_alert != 0)
  {
    prl_close();
    alert.prl_alert = 0;
  }
}//

void close_general_alarms(void)
{
  if (alert.cso_alert != 0)
  {
    cso_close();
    alert.cso_alert = 0;
  }

  if (alert.hso_alert != 0)
  {
    hso_close();
    alert.hso_alert = 0;
  }

  if (alert.fo_alert != 0)
  {
    fo_close();
    alert.fo_alert = 0;
  }

  if (alert.bi_alert != 0)
  {
    bi_close();
    alert.bi_alert = 0;
  }

  if (alert.csp_alert != 0)
  {
    csp_close();
    alert.csp_alert = 0;
  }

  if (alert.cs_alert != 0)
  {
    cs_close();
    alert.cs_alert = 0;
  }
}//close_general_alarms
void close_vital_air_alarms()
{
  if (alert.ath_alert != 0)
  {
    ath_close();
    alert.ath_alert = 0;
  }

  if (alert.atl_alert != 0)
  {
    atl_close();
    alert.atl_alert = 0;
  }

  if (alert.bto_t3_alert != 0)
  {
    bto_t3_close();
    alert.bto_t3_alert = 0;
  }
}//close_vital_air_alarms
void close_vital_skin_alarms()
{
  if (alert.bth_alert != 0)
  {
    bth_close();
    alert.bth_alert = 0;
  }

  if (alert.btl_alert != 0)
  {
    btl_close();
    alert.btl_alert = 0;
  }

  if (alert.btc_alert != 0)
  {
    btc_close();
    alert.btc_alert = 0;
  }

  if (alert.bto_t1_alert != 0)
  {
    bto_t1_close();
    alert.bto_t1_alert = 0;
  }
}//close_vital_skin_alarms
void close_vital_alarms()
{
  close_vital_air_alarms();
  close_vital_skin_alarms();
} //close_vital_alarms

int8_t ble_data_process(const char *ble_buff, uint8_t length)
{
  char buff[256];
  uint8_t len = length;

  uint8_t current_pos = 0;

  char did[7];
  did[6] = '\n';
  char mid[4];
  int mid_i = 0;
  //uint8_t bitt = 9;
  char buff1[256];

  //{"DID": "SBWXXX", "MID":"101", "ADT":"2", "AMT":"5" }

  memset(buff, 0, len);
  memcpy(buff, ble_buff, len);

  for (int i = 0; i < len; i++)
  {
    if (buff[i] == '"')
    {
      for (int j = i; j < len; j++)
      {
        buff[j] = buff[j + 1];
      }
    }
    len = strlen(buff);
  }

  len = strlen(buff);

  for (int i = 0; i < len; i++)
  {
    if (buff[i] == '}')
    {
      for (int j = i; j < len; j++)
      {
        buff[j] = '\n';
      }
      int k = 0;
    }
    len = strlen(buff);
  }

  len = strlen(buff);
  int lenn = len - 1;
  char str1[256];
  for (int i = 1; i < lenn; i++)
  {
    str1[i - 1] = buff[i];
  }

  typedef struct data
  {
    char *string;
    char *variable;
    char *value;
  }

  parsing;
  parsing parse[10];

  memset(parse, 0, sizeof(parse));

  parse[0].string = strtok(str1, ",");

  int c = 0;

  for (int i = 1; parse[i - 1].string != NULL; i++)
  {
    c = i;
    parse[i].string = strtok(NULL, ",");
  }

  for (int i = 0; i < c; i++)
  {
    parse[i].variable = strtok(parse[i].string, ":");
    parse[i].value = strtok(NULL, ":");
  }

  char tagg = *(parse[2].value);
  if (strstr(parse[0].variable, "NID"))
  {
    for (int j = 0; j < 6; j++)
      DID[j] = parse[0].value[j];
    return 1;
  }

  char *tDID = strstr(parse[0].value, DID);
  if (tDID == NULL)
  {
    uint8_t buff[128];
    memset(&buff, NULL, 128);
    sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"203\",\"AL\":\"DIDF\"}", DID); //DID Fail
    uint16_t length = (uint16_t) strlen(buff);
    ret_code_t ret;
    do {
      ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
    } while (ret != NRF_SUCCESS && ble_live != 0);
    nrf_delay_ms(100);
    NVIC_SystemReset();

    return 0;
  }
  else
  {
    did_check_flag = false;

    if (strstr(parse[1].value, "101"))  //mid
    {
      uint8_t pol = 0, cSP = 0, cPR = 0, Ladt = 0;
      Ladt = atoi(parse[2].value);

      if (Ladt >= 0 && Ladt <= 10)
      {
        adt = Ladt;

        cSP = abs(cal.spo * 10);
        cPR = abs(cal.prt * 10);

        if (cal.spo < 0)
          pol = pol | 0x01;
        if (cal.prt < 0)
          pol = pol | 0x02;

        //update_card();
        return 1;
      } //adt validation

    } //if MID 101
        else if (strstr(parse[1].value, "103")) //mid
    {
      uint8_t Lspl = 0, Lspg = 0, Lprl = 0, Lprg = 0;

      Lspl = atoi(parse[2].value);
      Lspg = atoi(parse[3].value);
      Lprl = atoi(parse[4].value);
      Lprg = atoi(parse[5].value);
      if (((Lspl >= 0) && (Lspl <= 250)) &&
          ((Lspg >= 0) && (Lspg <= 250)) &&
          ((Lprl >= 0) && (Lprl <= 250)) &&
          ((Lprg >= 0) && (Lprg <= 250)))
      {
        set.spo2_Ll = Lspl;
        set.spo2_Ul = Lspg;
        set.pRate_Ll = Lprl;
        set.pRate_Ul = Lprg;
        //update_card();
        return 1;
      } //validation
    } //mid = 103
    else if (strstr(parse[1].value, "104")) //mid
    {
      if (strstr(parse[2].variable, "TW"))
      {
        //Tear Weight Function
      }
      else
      {
        uint8_t cT1 = 0, cT2 = 0, cT3 = 0, cSP = 0, cPR = 0, pol = 0;
        cT1 = abs(cal.T1_skin * 10);
        cT2 = abs(cal.T2_peri * 10);
        cT3 = abs(cal.T3_air * 10);

        cPR = abs(cal.prt * 10);
        cSP = abs(cal.spo * 10);

        //if(cal.T1_skin < 0)
        //  pol = pol|0x01
        //if(cal.T2_peri < 0)
        //  pol = pol|0x02
        //if(cal.T3_air < 0)
        //  pol = pol|0x04

        if (strstr(parse[2].variable, "CPT"))
        {
          float cpt = atof(parse[2].value);
          if ((cpt <= 2.5) && (cpt >= -2.5))
          {
            cal.T2_peri = cpt;
            cT2 = abs(cal.T2_peri * 10);

            if (cal.T1_skin < 0)
              pol = pol | 0x01;
            if (cal.T2_peri < 0)
              pol = pol | 0x02;
            if (cal.T3_air < 0)
              pol = pol | 0x04;

          } //validation
        }
        else if (strstr(parse[2].variable, "CBT"))
        {
          float cbt = atof(parse[2].value);
          if ((cbt <= 2.5) && (cbt >= -2.5))
          {
            cal.T1_skin = cbt;
            cT1 = abs(cal.T1_skin * 10);

            if (cal.T1_skin < 0)
              pol = pol | 0x01;
            if (cal.T2_peri < 0)
              pol = pol | 0x02;
            if (cal.T3_air < 0)
              pol = pol | 0x04;

          } //validation
        }
        else if (strstr(parse[2].variable, "CAT"))
        {
          float cat = atof(parse[2].value);
          if ((cat <= 2.5) && (cat >= -2.5))
          {
            cal.T3_air = cat;
            cT3 = abs(cal.T3_air * 10);

            if (cal.T1_skin < 0)
              pol = pol | 0x01;
            if (cal.T2_peri < 0)
              pol = pol | 0x02;
            if (cal.T3_air < 0)
              pol = pol | 0x04;

          } //validation
        }
        else if (strstr(parse[2].variable, "CSPO"))
        {
          float csp = atof(parse[2].value);
          if ((csp <= 2.5) && (csp >= -2.5))
          {
            cal.spo = csp;
            cSP = abs(cal.spo * 10);

            if (cal.spo < 0)
              pol = pol | 0x01;
            if (cal.prt < 0)
              pol = pol | 0x02;

          } //validation
        }
        else if (strstr(parse[2].variable, "CPRT"))
        {
          float cpr = atof(parse[2].value);
          if ((cpr <= 2.5) && (cpr >= -2.5))
          {
            cal.prt = cpr;
            cPR = abs(cal.prt * 10);

            if (cal.spo < 0)
              pol = pol | 0x01;
            if (cal.prt < 0)
              pol = pol | 0x02;

          } //validation
        } //cprt
        else if (strstr(parse[2].variable, "CWT"))
        {
          float cwt = atof(parse[2].value);
          if ((cwt <= 2.500) && (cwt >= -2.500))
          {
            uint16_t cweight = 0;
            cal.wt = cwt;
            cweight = abs(cal.wt * 1000);
            if (cal.wt < 0.000)
              pol = pol | 0x01;
          }
        } //if weight calibration
          //update_card();
          return 1;
      } //else(other than TW)
    } //mid = 104
    else if (strstr(parse[1].value, "105"))
    {
      if (tagg == '0')  //Baby Discharged
      {
        //if (baby_status != DISCHARGED)
        //  F_sta.sBaby_status_flag = 1;

        baby_status = DISCHARGED;

        if (mode != MANUAL_MODE)
        {
          close_vital_alarms();
          close_general_alarms();
        }

        mode = MANUAL_MODE;

        set.htr = 0;
        real_mode = 9;
        real_htr = 0;

        
        NRF_LOG_INFO("baby discharge - Manual Mode-9");

        //F_sta.sMODE_flag = 1;
        //F_sta.sHTR_flag = 1;

        //if (fds_test_write(1, flash_W_1(set.T1 * 10, set.T3 * 10, set.htr)))
        //  F_sta.sHTR_flag = 2;
      }
      else
      {
        //if (baby_status != admitted)
        //  F_sta.sBaby_status_flag = 1;

        baby_status = ADMITTED;
      }
      Send_open_alarms();
      //update_card();
      return 1;
    } //mid105
    else if (strstr(parse[1].value, "106"))
    {
      float Lslv = atof(parse[2].value);
      if ((Lslv <= 5.0) && (Lslv >= 0.5) && (Lslv != slv))
      {
        slv = Lslv;
      } //validation

      float Lshv = atof(parse[3].value);
      if ((Lshv <= 5.0) && (Lshv >= 0.5))
      {
        shv = Lshv;
      } //validation

      float Lalv = atof(parse[4].value);
      if ((Lalv <= 5.0) && (Lalv >= 0.5))
      {
        alv = Lalv;
      } //validatio

      float Lahv = atof(parse[5].value);
      if ((Lahv <= 5.0) && (Lahv >= 0.5))
      {
        ahv = Lahv;
      } //validation
      //update_card();
      return 1;
    } //mid106
    else if (strstr(parse[1].value, "107"))
    {
      //*************** MID:107 ******************************************************************
      //{"DID":"SBWXXX", "MID":"107","ADT":"10","AMT":"20","SV":"22.5","BS":"1"}
      if (!did_check_flag)
      {
        uint8_t buff[128];
        memset(&buff, 0, 128);

        //if (mode == air_mode) {
        //  sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"107\",\"ADT\":\"%d\",\"SV\":\"%2.1f\",\"BS\":\"%d\"}", DID, adt, set.T3, baby_status);
        //}
        //else if (mode == skin_mode) {
        //  sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"107\",\"ADT\":\"%d\",\"SV\":\"%2.1f\",\"BS\":\"%d\"}", DID, adt, set.T1, baby_status);
        //}
        //else {
        //  sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"107\",\"ADT\":\"%d\",\"SV\":\"%d\",\"BS\":\"%d\"}", DID, adt, set.htr, baby_status);
        //}
        sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"107\",\"ADT\":\"%d\",\"SBV\":\"%2.1f\",\"SAV\":\"%2.1f\",\"BS\":\"%d\"}", DID, adt, set.T1, set.T3, baby_status);
        uint16_t length = (uint16_t) strlen(buff);
        ret_code_t ret;
        do {
          ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
        } while (ret != NRF_SUCCESS && ble_live != 0);
      }
    }//MID:107

    else if (strstr(parse[1].value, "108"))
    {
      //**************  MID:108 ****************************************************************************
      //{"DID":"SBWXXX", "MID":"108","CAT":"-5","CBT":"-8","CPT":"-4","CWT":"-50.000","CSP":"14","CPR":"13"}


      if (!did_check_flag)
      {
        uint8_t buff[128];
        memset(&buff, 0, 128);
        sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"108\",\"CAT\":\"%2.2f\",\"CBT\":\"%2.2f\",\"CPT\":\"%2.2f\",\"CWT\":\"%2.2f\",\"CSP\":\"%2.2f\",\"CPR\":\"%2.2f\"}", DID, cal.T3_air, cal.T1_skin, cal.T2_peri, cal.wt, cal.spo, cal.prt);
        uint16_t length = (uint16_t) strlen(buff);
        ret_code_t ret;
        do {
          ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
        } while (ret != NRF_SUCCESS && ble_live != 0);
      }
    }//MID:108

    else if (strstr(parse[1].value, "109"))
    {
      //************  MID:109 *****************************************************************************
      //{"DID":"SBWXXX", "MID":"109","SPL":"10","SPG":"20","PRL":"22.5","PRG":"22.5"}
      if (!did_check_flag)
      {
        uint8_t buff[128];
        memset(&buff, 0, 128);
        sprintf(buff, "{\"DID\":\"%s\",\"MID\":\"109\",\"SPL\":\"%d\",\"SPG\":\"%d\",\"PRL\":\"%d\",\"PRG\":\"%d\"}", DID, set.spo2_Ll, set.spo2_Ul, set.pRate_Ll, set.pRate_Ul);
        uint16_t length = (uint16_t) strlen(buff);
        ret_code_t ret;
        do {
          ret = ble_nus_data_send(&m_nus, buff, &length, m_conn_handle);
        } while (ret != NRF_SUCCESS && ble_live != 0);
      }
    }//MID:109

    else if (strstr(parse[1].value, "110"))
    {
      if (tagg == '1')
      {
        uint8_t pol = 0;
        nrf_gpio_pin_write(RELAY, 1);
        nrf_gpio_pin_write(NLED, 0);

        relayStatus = 1;
//        update_card();
        return 1;
      }
      if (tagg == '0')
      {
        uint8_t pol = 0;
        nrf_gpio_pin_write(RELAY, 0);
        nrf_gpio_pin_write(NLED, 1);

        relayStatus = 0;
        //update_card();
        return 1;
      }
    }//MID 110
    else if (strstr(parse[1].value, "111")) //mid
    {
      uint8_t Lhtr = atoi(parse[2].value);
      if (Lhtr >= 0 && Lhtr <= 100)
      {
        if (mode != MANUAL_MODE)
        {
          close_vital_alarms();
          close_general_alarms();
          blink_led = false;
          nrf_gpio_pin_write(LED, 0);
          led_offtime = app_timer_cnt_get();
        }
        real_mode = 9;
        NRF_LOG_INFO("Mode - Manual Mode-9");
        mode = MANUAL_MODE;
        hp_flag = 1;
        //////////////////////////
        set.htr = Lhtr;
        real_htr = Lhtr;
        //update_card();
        return 1;
      } //htr validation
    }//mid 111
    else if (strstr(parse[1].value, "112")) //mid
    {
      float sT1 = 0.0;
      sT1 = atof(parse[2].value);
      sT1 = sT1 - cal.T1_skin;

      if ((sT1 > 19.0 && sT1 <= 38.00) && (baby_status != DISCHARGED)) //skin_mode
      {
        if (mode != SKIN_MODE)
          close_vital_air_alarms();

        real_mode = SKIN_MODE;
        mode = SKIN_MODE;
        hp_flag = 1;
        set.T1 = sT1;

        set.Ul = set.T1 + 0.3;
        set.Ll = set.T1 - 0.3;

       // update_card();
      }//skin mode set Temp
    }//MID 112
    else if (strstr(parse[1].value, "113")) //mid
    {
      float sT3 = 0.0;
      sT3 = atof(parse[2].value);
      sT3 = sT3 - cal.T3_air;
      if ((sT3 > 19.0 && sT3 <= 39.00) && (baby_status != DISCHARGED)) //Air_mode
      {
        if (mode != AIR_MODE)
          close_vital_skin_alarms();

        real_mode = AIR_MODE;
        mode = AIR_MODE;
        hp_flag = 1;
        set.T3 = sT3;
        //set.T3 = (sT3 + cal.T3_air);
        //update_card();
        return 1;
      }
    }//MID 113
    else if (strstr(parse[1].value, "113")) //mid
    {      
      uint8_t Val = 0;
      Val = atoi(parse[2].value);
      if(Val == 1)
      {
        Send_open_alarms();
      }
    }

  }//else
} //ble_data_process
//********************************************************************************************************************************************************************************************************//


/**@brief Function for assert macro callback.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyse
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num    Line number of the failing ASSERT call.
 * @param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}

/**@brief Function for initializing the timer module.
 */
static void timers_init(void)
{
    ret_code_t err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);
}

/**@brief Function for the GAP initialization.
 *
 * @details This function will set up all the necessary GAP (Generic Access Profile) parameters of
 *          the device. It also sets the permissions and appearance.
 */
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *) DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling Queued Write Module errors.
 *
 * @details A pointer to this function will be passed to each service which may need to inform the
 *          application about an error.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void nrf_qwr_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for handling the data from the Nordic UART Service.
 *
 * @details This function will process the data received from the Nordic UART BLE Service and send
 *          it to the UART module.
 *
 * @param[in] p_evt       Nordic UART Service event.
 */
/**@snippet [Handling the data received over BLE] */

char ble_data_array[256];
char ble_data_array1[256];
char ble_data_array2[256];
char ble_data_array3[256];
uint8_t ble_data_flag = 0;
uint8_t ble_data_flag1 = 0;
uint8_t ble_data_flag2 = 0;
uint8_t ble_data_flag3 = 0;

static void nus_data_handler(ble_nus_evt_t * p_evt)
{

  if (p_evt->type == BLE_NUS_EVT_RX_DATA)
  {
      memset(ble_data_array, 0, sizeof(ble_data_array));
    if (ble_data_flag == 0)
    {
      for (uint32_t i = 0; i < p_evt->params.rx_data.length; i++)
      {
        ble_data_array[i] = p_evt->params.rx_data.p_data[i];
      }

      ble_data_flag = 1;
    }
    else if (ble_data_flag1 == 0)
    {
      for (uint32_t i = 0; i < p_evt->params.rx_data.length; i++)
      {
        ble_data_array1[i] = p_evt->params.rx_data.p_data[i];
      }

      ble_data_flag1 = 1;
    }
    else if (ble_data_flag2 == 0)
    {
      for (uint32_t i = 0; i < p_evt->params.rx_data.length; i++)
      {
        ble_data_array2[i] = p_evt->params.rx_data.p_data[i];
      }

      ble_data_flag2 = 2;
    }
    else if (ble_data_flag3 == 0)
    {
      for (uint32_t i = 0; i < p_evt->params.rx_data.length; i++)
      {
        ble_data_array3[i] = p_evt->params.rx_data.p_data[i];
      }

      ble_data_flag3 = 1;
    }
  }
}
/**@snippet [Handling the data received over BLE] */


/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
    uint32_t           err_code;
    ble_nus_init_t     nus_init;
    nrf_ble_qwr_init_t qwr_init = {0};

    // Initialize Queued Write Module.
    qwr_init.error_handler = nrf_qwr_error_handler;

    err_code = nrf_ble_qwr_init(&m_qwr, &qwr_init);
    APP_ERROR_CHECK(err_code);

    // Initialize NUS.
    memset(&nus_init, 0, sizeof(nus_init));

    nus_init.data_handler = nus_data_handler;

    err_code = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling an event from the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module
 *          which are passed to the application.
 *
 * @note All this function does is to disconnect. This could have been done by simply setting
 *       the disconnect_on_fail config parameter, but instead we use the event handler
 *       mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);
    }
}


/**@brief Function for handling errors from the Connection Parameters module.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for putting the chip into sleep mode.
 *
 * @note This function will not return.
 */
static void sleep_mode_enter(void)
{
    uint32_t err_code = bsp_indication_set(BSP_INDICATE_IDLE);
    APP_ERROR_CHECK(err_code);

    // Prepare wakeup buttons.
    err_code = bsp_btn_ble_sleep_mode_prepare();
    APP_ERROR_CHECK(err_code);

    // Go to system-off mode (this function will not return; wakeup will cause a reset).
    err_code = sd_power_system_off();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling advertising events.
 *
 * @details This function will be called for advertising events which are passed to the application.
 *
 * @param[in] ble_adv_evt  Advertising event.
 */
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
    uint32_t err_code;

    switch (ble_adv_evt)
    {
        case BLE_ADV_EVT_FAST:
            err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
            APP_ERROR_CHECK(err_code);
            break;
        case BLE_ADV_EVT_IDLE:
            sleep_mode_enter();
            break;
        default:
            break;
    }
}


/**@brief Function for handling BLE events.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 * @param[in]   p_context   Unused.
 */
static void ble_evt_handler(ble_evt_t const * p_ble_evt, void * p_context)
{
    uint32_t err_code;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            NRF_LOG_INFO("Connected");
            err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
            APP_ERROR_CHECK(err_code);
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            err_code = nrf_ble_qwr_conn_handle_assign(&m_qwr, m_conn_handle);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GAP_EVT_DISCONNECTED:
            NRF_LOG_INFO("Disconnected");
            // LED indication will be changed when advertising starts.
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            break;

        case BLE_GAP_EVT_PHY_UPDATE_REQUEST:
        {
            NRF_LOG_DEBUG("PHY update request.");
            ble_gap_phys_t const phys =
            {
                .rx_phys = BLE_GAP_PHY_AUTO,
                .tx_phys = BLE_GAP_PHY_AUTO,
            };
            err_code = sd_ble_gap_phy_update(p_ble_evt->evt.gap_evt.conn_handle, &phys);
            APP_ERROR_CHECK(err_code);
        } break;

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            // Pairing not supported
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_SYS_ATTR_MISSING:
            // No system attributes have been stored.
            err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_TIMEOUT:
            // Disconnect on GATT Server timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for the SoftDevice initialization.
 *
 * @details This function initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    ret_code_t err_code;

    err_code = nrf_sdh_enable_request();
    APP_ERROR_CHECK(err_code);

    // Configure the BLE stack using the default settings.
    // Fetch the start address of the application RAM.
    uint32_t ram_start = 0;
    err_code = nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start);
    APP_ERROR_CHECK(err_code);

    // Enable BLE stack.
    err_code = nrf_sdh_ble_enable(&ram_start);
    APP_ERROR_CHECK(err_code);

    // Register a handler for BLE events.
    NRF_SDH_BLE_OBSERVER(m_ble_observer, APP_BLE_OBSERVER_PRIO, ble_evt_handler, NULL);
}


/**@brief Function for handling events from the GATT library. */
void gatt_evt_handler(nrf_ble_gatt_t * p_gatt, nrf_ble_gatt_evt_t const * p_evt)
{
    if ((m_conn_handle == p_evt->conn_handle) && (p_evt->evt_id == NRF_BLE_GATT_EVT_ATT_MTU_UPDATED))
    {
        m_ble_nus_max_data_len = p_evt->params.att_mtu_effective - OPCODE_LENGTH - HANDLE_LENGTH;
        NRF_LOG_INFO("Data len is set to 0x%X(%d)", m_ble_nus_max_data_len, m_ble_nus_max_data_len);
    }
    NRF_LOG_DEBUG("ATT MTU exchange completed. central 0x%x peripheral 0x%x",
                  p_gatt->att_mtu_desired_central,
                  p_gatt->att_mtu_desired_periph);
}


/**@brief Function for initializing the GATT library. */
void gatt_init(void)
{
    ret_code_t err_code;

    err_code = nrf_ble_gatt_init(&m_gatt, gatt_evt_handler);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_gatt_att_mtu_periph_set(&m_gatt, NRF_SDH_BLE_GATT_MAX_MTU_SIZE);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated by button press.
 */
void bsp_event_handler(bsp_event_t event)
{
    uint32_t err_code;
    switch (event)
    {
        case BSP_EVENT_SLEEP:
            sleep_mode_enter();
            break;

        case BSP_EVENT_DISCONNECT:
            err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            if (err_code != NRF_ERROR_INVALID_STATE)
            {
                APP_ERROR_CHECK(err_code);
            }
            break;

        case BSP_EVENT_WHITELIST_OFF:
            if (m_conn_handle == BLE_CONN_HANDLE_INVALID)
            {
                err_code = ble_advertising_restart_without_whitelist(&m_advertising);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
            }
            break;

        default:
            break;
    }
}


/**@brief   Function for handling app_uart events.
 *
 * @details This function will receive a single character from the app_uart module and append it to
 *          a string. The string will be be sent over BLE when the last character received was a
 *          'new line' '\n' (hex 0x0A) or if the string has reached the maximum data length.
 */
/**@snippet [Handling the data received over UART] */
void uart_event_handle(app_uart_evt_t * p_event)
{
    static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
    static uint8_t index = 0;
    uint32_t       err_code;

    switch (p_event->evt_type)
    {
        case APP_UART_DATA_READY:
            //UNUSED_VARIABLE(app_uart_get(&data_array[index]));
            //index++;

            //if ((data_array[index - 1] == '\n') ||
            //    (data_array[index - 1] == '\r') ||
            //    (index >= m_ble_nus_max_data_len))
            //{
            //    if (index > 1)
            //    {
            //        NRF_LOG_DEBUG("Ready to send data over BLE NUS");
            //        NRF_LOG_HEXDUMP_DEBUG(data_array, index);

            //        do
            //        {
            //            uint16_t length = (uint16_t)index;
            //            err_code = ble_nus_data_send(&m_nus, data_array, &length, m_conn_handle);
            //            if ((err_code != NRF_ERROR_INVALID_STATE) &&
            //                (err_code != NRF_ERROR_RESOURCES) &&
            //                (err_code != NRF_ERROR_NOT_FOUND))
            //            {
            //                APP_ERROR_CHECK(err_code);
            //            }
            //        } while (err_code == NRF_ERROR_RESOURCES);
            //    }

            //    index = 0;
            //}
            break;

        case APP_UART_COMMUNICATION_ERROR:
            //APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            //APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
}
/**@snippet [Handling the data received over UART] */


/**@brief  Function for initializing the UART module.
 */
/**@snippet [UART Initialization] */
#define pin_rx0 NRF_GPIO_PIN_MAP(1, 03)
#define pin_tx0 NRF_GPIO_PIN_MAP(0, 11)
static void uart_init(void)
{
    uint32_t                     err_code;
    app_uart_comm_params_t const comm_params =
    {
        .rx_pin_no    = pin_rx0,//RX_PIN_NUMBER,
        .tx_pin_no    = pin_tx0,//TX_PIN_NUMBER,
        .rts_pin_no   = RTS_PIN_NUMBER,
        .cts_pin_no   = CTS_PIN_NUMBER,
        .flow_control = APP_UART_FLOW_CONTROL_DISABLED,
        .use_parity   = false,
#if defined (UART_PRESENT)
        .baud_rate    = NRF_UART_BAUDRATE_115200
#else
        .baud_rate    = NRF_UARTE_BAUDRATE_115200
#endif
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOWEST,
                       err_code);
    APP_ERROR_CHECK(err_code);
}
/**@snippet [UART Initialization] */


/**@brief Function for initializing the Advertising functionality.
 */
static void advertising_init(void)
{
    uint32_t               err_code;
    ble_advertising_init_t init;

    memset(&init, 0, sizeof(init));

    init.advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    init.advdata.include_appearance = false;
    init.advdata.flags              = BLE_GAP_ADV_FLAGS_LE_ONLY_LIMITED_DISC_MODE;

    init.srdata.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    init.srdata.uuids_complete.p_uuids  = m_adv_uuids;

    init.config.ble_adv_fast_enabled  = true;
    init.config.ble_adv_fast_interval = APP_ADV_INTERVAL;
    init.config.ble_adv_fast_timeout  = APP_ADV_DURATION;
    init.evt_handler = on_adv_evt;

    err_code = ble_advertising_init(&m_advertising, &init);
    APP_ERROR_CHECK(err_code);

    ble_advertising_conn_cfg_tag_set(&m_advertising, APP_BLE_CONN_CFG_TAG);
}


/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
static void buttons_leds_init(bool * p_erase_bonds)
{
    bsp_event_t startup_event;

    uint32_t err_code = bsp_init(BSP_INIT_LEDS | BSP_INIT_BUTTONS, bsp_event_handler);
    APP_ERROR_CHECK(err_code);

    err_code = bsp_btn_ble_init(NULL, &startup_event);
    APP_ERROR_CHECK(err_code);

    *p_erase_bonds = (startup_event == BSP_EVENT_CLEAR_BONDING_DATA);
}


/**@brief Function for initializing the nrf log module.
 */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}


/**@brief Function for initializing power management.
 */
static void power_management_init(void)
{
    ret_code_t err_code;
    err_code = nrf_pwr_mgmt_init();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the idle state (main loop).
 *
 * @details If there is no pending log operation, then sleep until next the next event occurs.
 */
static void idle_state_handle(void)
{
    UNUSED_RETURN_VALUE(NRF_LOG_PROCESS());
    nrf_pwr_mgmt_run();
}


/**@brief Function for starting advertising.
 */
static void advertising_start(void)
{
    uint32_t err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
    APP_ERROR_CHECK(err_code);
}


/**@brief Application main function.
 */
int main(void)
{
    bool erase_bonds;

    // Initialize.
    uart_init();
    log_init();
      NRF_LOG_INFO("PROGRAM STARTED");

    int32_t reset_reason = NRF_POWER->RESETREAS;
    NRF_LOG_INFO("Reset reason = 0x%08x.\n", reset_reason);
    NRF_POWER->RESETREAS = NRF_POWER->RESETREAS;

    timers_init();
    buttons_leds_init(&erase_bonds);
    power_management_init();
    ble_stack_init();
    gap_params_init();
    gatt_init();
    services_init();
    advertising_init();
    conn_params_init();

    // Start execution.
    //printf("\r\nUART started.\r\n");
    NRF_LOG_INFO("Debug logging for UART over RTT started.");
    advertising_start();

    // Enter main loop.
    for (;;)
    {
      if (((5 * 60 * 1000) <= (app_timer_ms((app_timer_cnt_get()) - did_check_ticks))) && did_check_flag)
      {
        NVIC_SystemReset();
      }
      if (ble_data_flag)
      {
        ble_data_process(ble_data_array, strlen(ble_data_array));
        ble_data_flag = 0;
      }
      else if (ble_data_flag1)
      {
        ble_data_process(ble_data_array1, strlen(ble_data_array1));
        ble_data_flag1 = 0;
      }
      else if (ble_data_flag2)
      {
        ble_data_process(ble_data_array2, strlen(ble_data_array2));
        ble_data_flag3 = 0;
      }
      else if (ble_data_flag3)
      {
        ble_data_process(ble_data_array3, strlen(ble_data_array3));
        ble_data_flag3 = 0;
      }

    /////////////////////////////////////////////////////////////////////////////////
        htr_control(); 
        nrf_delay_ms(250);
    }//while
}//main


void htr_control(void)
{
 char buff[80];
 memset(buff,0,sizeof(buff));
 switch(mode)
 {
   case SKIN_MODE:
      Actual.htr = htr_percentage(set.T1,Actual.T1);
      sprintf(buff,"%uH",Actual.htr);
      send_warm2(buff,strlen(buff));
    break;
   case AIR_MODE:
      Actual.htr = htr_percentage(set.T3,Actual.T3);
      sprintf(buff,"%uH",Actual.htr);
      send_warm2(buff,strlen(buff));
    break;
   case MANUAL_MODE:
      Actual.htr = set.htr;
      sprintf(buff,"%uH",set.htr);
      //NRF_LOG_INFO("%s-%d",buff,strlen(buff));
      send_warm2(buff,strlen(buff));
    break;
   default:
   break;
 }
}
uint8_t htr_percentage(float setTemp, float actualTemp)
{
float diff_temp = setTemp - actualTemp;

if(diff_temp>=0.6)
  return 100;

if((diff_temp>=0.5) && (diff_temp<0.6))
  return 80;

if((diff_temp>=0.4) && (diff_temp<0.5))
  return 70;

if((diff_temp>=0.3) && (diff_temp<0.4))
  return 50;

if((diff_temp>=0.2) && (diff_temp<0.3))
  return 30;

if((diff_temp>=0.1) && (diff_temp<0.2))
  return 10;

if(diff_temp<0.1)
  return 0;
}
void send_warm2(uint8_t * str,size_t length)
{
  ret_code_t err_code;
  //NRF_LOG_RAW_INFO("%s-%d",str,length);
  
  for(uint8_t k=0; k<length; k++)
  {
    app_uart_put(str[k]);
  }
 
}